package main

import (
	"bufio"
	"fmt"
	"io"
	"net"
	"os"
	"sync"
	"strings"
	"time"
)


func connectToRedis(host, password string) (conn net.Conn) {
	var err error
	conn, err = net.DialTimeout("tcp", host, 5 * time.Second /*timeout*/)
	if err != nil {
		panic(err)
	}
	_, err = conn.Write([]byte("AUTH " + password + "\r\n"))
	if err != nil {
		panic(err)
	}
	return conn
}

func NewAllInterfaceConns() []net.Conn {
	fi, err := os.Open("proxyInfo")
	if err != nil {
		fmt.Printf("Error: %s\n", err)
		return nil
	}
	defer fi.Close()

	br := bufio.NewReader(fi)
	pwd, _, _ := br.ReadLine()

	conns := []net.Conn{}

	for {
		line, _, c := br.ReadLine()
		if c == io.EOF {
			break
		}
		fileds := strings.Fields(string(line))
		addr := fileds[0] + ":" + fileds[1]
		conn := connectToRedis(addr, string(pwd))
		conns = append(conns, conn)
	}
	return conns
}

// monitor starts outout commands in monitor
func monitor(conn net.Conn, wg *sync.WaitGroup) {
	buf := bufio.NewReader(conn)
	conn.Write([]byte("monitor" + "\r\n"))

	for {
		str, err := buf.ReadString('\n')

		if len(str) > 0 {
			if str == "+OK\r\n" {
				continue
			}

			if strings.HasPrefix(str, "-ERR ") {
				fmt.Printf("Redis return error message: " + strings.TrimRight(str[1:], "\r\n"))
				wg.Done()
			}

			fmt.Printf("%s", str[1:])
		}

		if err != nil {
			fmt.Println(err.Error())
			wg.Done()
		}
	}
}

func main() {
	var wg sync.WaitGroup
	conns := NewAllInterfaceConns()
	for _, tmpconn := range conns {
		wg.Add(1)
		conn := tmpconn
		go monitor(conn, &wg)
	}
	wg.Wait()
}

